package com.vandyo.sentry.core.cases;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.vandyo.sentry.core.dto.ErrStatus;
import com.vandyo.sentry.core.dto.Res;
import com.vandyo.sentry.core.exceptions.CaseException;
import com.vandyo.sentry.core.tools.Signature;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 轨迹行驶行为
 */
public class CarLocusEventDecorator extends CaseDecorator {

    private static final String carLocusEventUrl = "/car/api/locus/event/v111";

    public CarLocusEventDecorator(Case aCase) {
        super(aCase);
    }

    @Override
    public Res<Map<String, String>> check() throws CaseException {
        Res<Map<String, String>> oldRes = super.check();
        if (oldRes.getSuccess()) {
            Map<String, String> carMap = oldRes.getData();
            Res<Map<String, String>> newRes = new Res<>();
            newRes.setSId(oldRes.getSId());
            newRes.setUId(oldRes.getUId());
            if (Objects.isNull(carMap) || !carMap.containsKey("cid")) {
                newRes.setIgnore(true);
                newRes.setStatus(ErrStatus.ErrUnexpected);
                newRes.setSuccess(false);
            } else {
                Map<String, String> paramMap = new HashMap<>();
                paramMap.put("cid",carMap.get("cid"));
                paramMap.put("uid",oldRes.getUId());
                paramMap.put("start",carMap.get("sti"));
                paramMap.put("end",carMap.get("eti"));
                try {
                    HttpResponse response = HttpRequest.get(
                            Signature.host + carLocusEventUrl + "?"
                                    + Signature.getUrlParamsByMap(Signature.sign(paramMap, oldRes.getSId()))
                    ).execute();
                    if (response.isOk()) {
                        JSONObject jsonObject = JSONUtil.parseObj(response.body());
                        newRes.setData(carMap);
                        newRes.setSuccess(true);
                    } else {
                        newRes.setKey(carLocusEventUrl);
                        newRes.setStatus(Signature.matchStatus(response.getStatus()));
                        newRes.setSuccess(false);
                    }
                } catch (Exception e) {
                    newRes.setKey(carLocusEventUrl);
                    newRes.setStatus(ErrStatus.ErrUnknown);
                    newRes.setSuccess(false);
                }
            }
            return newRes;
        } else {
            oldRes.setIgnore(true);
            return oldRes;
        }
    }
}
